[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Global Filings client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://developer.factset.com/api-catalog/global-filings-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.GlobalFilings)](https://pypi.org/project/fds.sdk.GlobalFilings/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)


The Filings API provides a comprehensive set of functionalities for searching and retrieving filing documents from various sources.

This API is designed to expose document search and document retrieval functionality. A document search will allow a request to query FactSet document databases based on certain search criteria specified in the request.

In addition, a document count request will provide information on the number of stories available for a particular security.

The reference endpoints, such as sources, formTypes, timeZones, and categories, provide comprehensive lists of available sources, form types, time zones, and categories, respectively.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 1.0.3
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.GlobalFilings==1.0.3
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.GlobalFilings==1.0.3
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.GlobalFilings
from fds.sdk.GlobalFilings.api import filings_api_api
from fds.sdk.GlobalFilings.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.GlobalFilings.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.GlobalFilings.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.GlobalFilings.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = filings_api_api.FilingsAPIApi(api_client)
    sources = ["EDG"] # [str] | Code for document source to include.This is a comma-separated list. Use the ```/reference/sources``` endpoint to get the list of available sources.  
    start_date = "20231222" # str | Start Date. Format is YYYYMMDD or relative +/- days (0,-1,etc).  **Note:** **The API supports data from 1995 onwards. Ensure that the provided Date falls within this range for accurate results.**  (optional) if omitted the server will use the default value of "0"
    end_date = "20231223" # str | End Date. Format is YYYYMMDD or relative +/- days (0,-1,etc). (optional) if omitted the server will use the default value of "0"
    ids = ["MODN-US"] # [str] | Requested symbols or securities.  This is a comma-separated list with a maximum limit of 1000.  Each symbol can be a FactSet exchange symbol, CUSIP, or SEDOL. (optional)

    try:
        # Returns the count of filings for specified source.
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_count(sources, start_date=start_date, end_date=end_date, ids=ids)

        pprint(api_response)
    except fds.sdk.GlobalFilings.ApiException as e:
        print("Exception when calling FilingsAPIApi->get_count: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns the count of filings for specified source.
    #     api_response, http_status_code, response_headers = api_instance.get_count_with_http_info(sources, start_date=start_date, end_date=end_date, ids=ids)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.GlobalFilings.ApiException as e:
    #     print("Exception when calling FilingsAPIApi->get_count: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns the count of filings for specified source.
    #     async_result = api_instance.get_count_async(sources, start_date=start_date, end_date=end_date, ids=ids)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.GlobalFilings.ApiException as e:
    #     print("Exception when calling FilingsAPIApi->get_count: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns the count of filings for specified source.
    #     async_result = api_instance.get_count_with_http_info_async(sources, start_date=start_date, end_date=end_date, ids=ids)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.GlobalFilings.ApiException as e:
    #     print("Exception when calling FilingsAPIApi->get_count: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.GlobalFilings

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.GlobalFilings.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.GlobalFilings

configuration = fds.sdk.GlobalFilings.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.GlobalFilings

configuration = fds.sdk.GlobalFilings.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.GlobalFilings

configuration = fds.sdk.GlobalFilings.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/global-filings/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*FilingsAPIApi* | [**get_count**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/FilingsAPIApi.md#get_count) | **GET** /count | Returns the count of filings for specified source.
*FilingsAPIApi* | [**get_filings**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/FilingsAPIApi.md#get_filings) | **GET** /search | Returns the filings documents and related metadata within FactSet coverage.
*FilingsAPIApi* | [**get_formtype**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/FilingsAPIApi.md#get_formtype) | **GET** /reference/form-types | Returns the form types of EDGAR.
*FilingsAPIApi* | [**get_sources**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/FilingsAPIApi.md#get_sources) | **GET** /reference/sources | Returns the sources.
*FilingsAPIApi* | [**get_timezone**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/FilingsAPIApi.md#get_timezone) | **GET** /reference/time-zones | Returns the time zones.
*FilingsAPIApi* | [**getcategories**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/FilingsAPIApi.md#getcategories) | **GET** /reference/categories | Returns the categories.


## Documentation For Models

 - [ErrorExample](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ErrorExample.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ErrorResponse.md)
 - [InvestmentResearch](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/InvestmentResearch.md)
 - [InvestmentResearchMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/InvestmentResearchMeta.md)
 - [InvestmentResearchMetaPagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/InvestmentResearchMetaPagination.md)
 - [InvestmentResearchObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/InvestmentResearchObject.md)
 - [ResponseCategies](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseCategies.md)
 - [ResponseCategoriesObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseCategoriesObject.md)
 - [ResponseCount](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseCount.md)
 - [ResponseCountObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseCountObject.md)
 - [ResponseFormtype](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseFormtype.md)
 - [ResponseFormtypeObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseFormtypeObject.md)
 - [ResponseSources](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseSources.md)
 - [ResponseSourcesObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseSourcesObject.md)
 - [ResponseTime](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseTime.md)
 - [ResponseTimeObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/GlobalFilings/v1/docs/ResponseTimeObject.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.GlobalFilings.apis and fds.sdk.GlobalFilings.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.GlobalFilings.api.default_api import DefaultApi`
- `from fds.sdk.GlobalFilings.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.GlobalFilings
from fds.sdk.GlobalFilings.apis import *
from fds.sdk.GlobalFilings.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

